/*****************************************************************************
     This file is part of the Changedfiles system. 

     Copright (C)2002  Michael L. Welles

     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*******************************************************************************/
/*  $Id: action.cpp,v 1.10 2003/11/10 20:42:01 mike Exp $ */
#include <string> 
#include <iostream>
#include <stdio.h> 

#include "action.h"

using std::cerr;

Action::Action(){
  m_retry = 0;
  m_timeout_retry = 0;
  m_max_timeout_retry = 0;
}

bool Action::first(){
  cerr << "Add initialization code";
  return false;
}

bool Action::last(){
  cerr << "Add wrap-up code" ;
  return false;
}

/** 
 * returns number of retrys left.
 */
int Action::add_retry()
{ 
  if ( m_max_retry == 0 )
    {
      return 0;
    }
  //If max_retry is -1, then we should loop infinatly.
  if ( m_max_retry < 0 )
    { 
      Logger * logger = Logger::get_instance();
      logger->log( Logger::INFO, "info - Retry for rule set to -1 or less.  Action will loop infinatly on failure.");
      return 1;
    }
  int trys_left = 0;
  if( m_max_retry > 0 )
    {
      trys_left = ( m_max_retry - m_retry );
      if ( trys_left < 0 ){
	trys_left = 0;
      }
    }
  else
    {
      trys_left = m_retry;
    }
  ++m_retry;
  return trys_left;
} 
int Action::add_timeout_retry()
{ 
  Logger * logger = Logger::get_instance();

  //If max_retry is -1, then we should loop infinatly.
  if ( m_max_timeout_retry < 0 )
    { 
      logger->log( Logger::INFO, "info - Timeout retry for rule set to -1 or less.  Action will loop infinatly on failure.");
      return 1;
    }
  ++m_timeout_retry;
  int trys_left = 0;
  if( m_timeout_retry > 0 )
    {
      trys_left = ( m_max_timeout_retry - m_timeout_retry );
      if ( trys_left < 0 ){
	trys_left = 0;
      }
    }
  else
    {
      trys_left = m_timeout_retry;
    }
  

  if ( trys_left > 0 ) 
    { 
      char message[ 500 ];
      sprintf( message, "After timeout, if file continues to be open, action will be taken %d more times before moving on with the queue.  (%d remaining attempts of %d max TIMEOUT_RETRY defined) ", trys_left, trys_left, m_max_timeout_retry );
      logger->log( Logger::INFO, message );
    }
  return trys_left;
} 

int Action::get_retry()
{ 
  return m_retry; 
} 

int Action::get_max_retry()
{ 
  return m_max_retry; 
}

bool Action::set_operation( FileOperation file_operation )
{
  m_file_operation = file_operation;
  return true;
}

bool Action::is_ready()
{
  return m_file_operation.is_ready();
}
  
bool Action::is_timeout()
{
  return m_file_operation.is_timeout();
}

bool Action::set_command_timeout( time_t timeout )
{
  m_command_timeout = timeout;
  return true;
}
