/**
 * Abstract base class for all actions changedfiles should
 * take on a file action notification.
 */

/**************************************************************************
 * This file is part of the Changedfiles system. Copright (C)2002  Micheal Welles
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _ACTION_H_
#define _ACTION_H_
//#include "action_exception.h"
#include "file_operation.h"
#include <string>

using std::string;

class Action {
 public:
  Action();

  virtual ~Action() { };

  virtual bool first();
  virtual bool last();
  virtual int get_retry();
  void set_max_retry(int max_retry){ this->m_max_retry = max_retry; }
  void set_timeout_retry( int timeout_retry){ this->m_max_timeout_retry =  timeout_retry; }
  
  void set_timeout( time_t timeout ) { this->m_timeout = timeout; }
  bool set_command_timeout( time_t timeout );
  time_t get_command_timeout() { return this->m_command_timeout; }

  bool set_operation( FileOperation file_operation );
  bool is_ready();
  bool is_timeout();
  int add_retry();
  int add_timeout_retry();

  virtual int get_max_retry();

  // no longer throws actionException
  virtual bool do_action() = 0;
  virtual string get_description() = 0;


 protected:

  int m_max_retry; 
  int m_retry;  
  int m_max_timeout_retry;
  int m_timeout_retry;
  time_t m_timeout;
  time_t m_command_timeout;

 private:
  FileOperation m_file_operation;

};

#endif
