/*****************************************************************************
     This file is part of the changedfiles system. 

     Copright (C)2001  Michael L. Welles

     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*******************************************************************************/
/*  $Id: bangconfig.cpp,v 1.19 2003/12/02 00:33:33 mike Exp $ */
#include <string> 
#include <stdio.h>
#include <regex.h>
#include "bangconfig.h"
#include <fstream>
#include "operations.h"

using std::string; 
using std::ifstream; 


BangConf::BangConf() 
{ 
  m_logger = Logger::get_instance(); 
  m_valid = false; 
}

BangConf::BangConf( const string& filename )
{
  if( ! read_config( filename.c_str()) ){
    m_logger->log( Logger::ERR, m_error );
  }
}  

bool BangConf::read_config( const string& filename) 
{ 
  return read_config( filename.c_str()); 
}


bool BangConf::read_config( const char * filename ) 
{ 
  string::size_type pos; 
  char line_buffer[M_READ_BUF_SIZE]; 
  string cur_rule_str;
  m_logger = Logger::get_instance(); 

  ifstream is( filename ); 
  if ( !is.good() ) 
    { 
      m_error = "BangConf::read_config() error: Failed to open "; 
      m_error += filename; 
      return false; 
    }
  
  
  while ( is.getline( line_buffer, M_READ_BUF_SIZE ) )
    { 
      //ignore blank lines
      if ( strlen( line_buffer ) == 0 ) 
	{
	  continue; 
	}
      //ignore comments
      else if ( line_buffer[ 0 ] == '#' ) 
	{ 
	  continue; 
	}

      string line = line_buffer; 
      pos = line.find_first_of( " \t" ); 
      if ( pos == string::npos ) 
	{ 
	  m_error = "Bad config! Error reading config file.  Offending line: '"; 
	  m_error += line_buffer; 
	  m_error += "'";
	  return false; 
	}
      string op_str = line.substr( 0, pos ); 
      pos = line.find_first_not_of(" \t", pos ); 
      if ( pos == string::npos ) 
	{ 
	  m_error = "Bad config! Error reading config file.  Offending line: '"; 
	  m_error += line_buffer; 
	  m_error += "'";
	  return false; 
	}
      string arg = line.substr( pos ); 

      if ( op_str == "RULE" ) 
	{ 
	  Rule* rule = new Rule(); 
	  regex_t preg; 

	  cur_rule_str = arg; 
	  rule->set_rule_str( cur_rule_str ); 

	  regcomp(&preg, cur_rule_str.c_str(), REG_EXTENDED); 
	  rule->set_regexp( preg ); 
	    
	  m_rule_map[ cur_rule_str ] = rule; 

	}
      else 
	{ 
	  // make sure we're in a rule
	  if ( cur_rule_str.length() != 0 ) 
	    { 
	      string line = line_buffer; 
	      pos = line.find_first_of( " \t" ); 
	      
	      Rule* rule = m_rule_map[ cur_rule_str ]; 

	      if ( op_str == "IGNORE" ) 
		{ 
		  rule->add_ignore( arg ); 
		  m_rule_map[ cur_rule_str ] = rule; 
		}
	      
	      else if ( op_str == "RETRY" ) 
		{ 
		  rule->set_retry( atoi( arg.c_str() ) ); 
		  m_rule_map[ cur_rule_str ] = rule; 
		}
	      else if ( op_str == "TIMEOUT" ) 
		{ 
		  rule->set_timeout( atoi( arg.c_str() ) ); 
		  m_rule_map[ cur_rule_str ] = rule; 
		}
	      else if ( op_str == "TIMEOUT_RETRY" )
		{
		  rule->set_timeout_retry( atoi( arg.c_str() ) );
		  m_rule_map[ cur_rule_str ] = rule;
		}
	      else if ( op_str == "COMMAND_TIMEOUT" )
		{
		  rule->set_command_timeout( atoi( arg.c_str() ) );
		  m_rule_map[ cur_rule_str ] = rule;
		}
	      else 
		{ 
		  e_optype op = parse_op_str( op_str ); 	    
		  
		  if ( op != NOOP )
		    {
		      rule->set_rule_str( op, arg); 
		      m_rule_map[ cur_rule_str ] = rule; 
		    }
		}
	    }
	}
    } // end while

  //figure out if we were sorta successful
  int numrules = 0;
  for (RULE_MAP::iterator p=m_rule_map.begin(); p != m_rule_map.end(); ++p)
    { 
      m_valid = true; 
      numrules++;
    }

  char loaded[38];
  sprintf( loaded, "Config file read: Rules loaded: %d", numrules );
  m_logger->log( Logger::INFO, loaded  );

  

  return m_valid; 
}

RULE_MAP BangConf::get_rules() 
{ 
  return m_rule_map; 
}

// extract regexp rule from enclosing brackets
string BangConf::parse_rule( const char * line ) 
{

  string::size_type pos; 
  string rule = line; 
  // skip past leading [
  rule = rule.substr( 1 ); 
  // find ending ]; 
  pos = rule.find_last_of( "]" ); 
  if ( pos != string::npos ) 
    rule = rule.substr( 0, pos ); 
  return rule; 
}
  

e_optype BangConf::parse_op_str( const string& op_str )
{ 
  e_optype op; 
  if ( op_str == "OPENW" ) 
    op = OPENW; 
  else if ( op_str == "OPENR" ) 
    op = OPENR; 
  else if ( op_str == "MKDIR" )
    op = MKDIR; 
  else if ( op_str == "RMDIR" )
    op = RMDIR;       
  else if ( op_str == "UNLINK" )
    op = UNLINK; 
  else if ( op_str == "SYMLINK" )
    op = SYMLINK; 
  else if ( op_str == "CREATE" )
    op = CREATE; 
  else if ( op_str == "LINK" )
    op = LINK; 
  else if ( op_str == "CLOSE" )
    op = CLOSE; 
  else if ( op_str == "RENAME" )
    op = RENAME; 
  else if ( op_str == "CHOWN" )
    op = CHOWN;
  else if ( op_str == "CHMOD" )
    op = CHMOD;
  else 
    op = NOOP; 
  return op; 
}

      
string BangConf::get_error() 
{
  return m_error; 
}


  
bool BangConf::is_valid()
{ 
  return m_valid; 
}
