/*****************************************************************************
     This file is part of the Changedfiles system. 

     Copright (C)2001  Michael L. Welles

     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*******************************************************************************/
/*  $Id: bangreader.cpp,v 1.32 2003/11/10 21:58:43 mike Exp $ */
#include "bangreader.h"
#include "queue_handler.h"
#include "file_operation.h"

#include <string> 
#include <stdio.h> 
#include <sys/types.h>
#include <fcntl.h>
#include <error.h> 
#include <errno.h>
#include <unistd.h> 
#include <time.h>
#include <regex.h>
#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h> 
#include <pthread.h>

using std::string; 

namespace 
{ 
  extern "C" void *reader_thread_startup(void *) ; 
  void *reader_thread_startup(void *_reader)
  { 
    BangReader *reader = (BangReader *)_reader; 
    reader->check_loop(); 
    return reader;
  }

  string _changedfiles_seperator = "]]]----->>"; 

  pthread_mutex_t m_list_mutex = PTHREAD_MUTEX_INITIALIZER; 
  int REST_INTERVAL = 0; 

} 

BangReader::BangReader(QueueHandler* que)
{
  m_logger = Logger::get_instance(); 
  m_bufsize=512; 
  m_device="/dev/changedfiles"; 
  m_fd = -1; 
  set_que( que );
  m_logger->log( Logger::DEBUG, "Module listener started normally" );
}

BangReader::BangReader(QueueHandler* que, string device)
{ 
  m_logger = Logger::get_instance(); 
  m_bufsize=512; 
  m_device=device; 
  m_fd = -1; 
  set_que( que );
  m_logger->log( Logger::DEBUG, "Module listener started normally for " + device );
}

BangReader::~BangReader()
{ 
  if ( m_fd >= 0 )
    { 
      close( m_fd ); 
    }
}
    
bool BangReader::set_que( QueueHandler* que )
{ 
  m_que = que;
  return true;
}

int BangReader::read_new()
{ 
  int i; 
  int numread=0; 
  char msg[m_bufsize]; 
  m_logger = Logger::get_instance();

  while ( 1 ) 
    {
      if ( m_fd < 0 ) 
	{ 
	  m_fd=open(m_device.c_str(), O_RDONLY); 
	  if ( m_fd < 0 ) 
	    { 
	      m_error="error - Error! Could not open "; 
	      m_error += m_device; 
	      m_error +=  " :" ; 
	      m_error += strerror(errno); 
	      m_logger->log( Logger::ERR, m_error ); 
	    }
	}
      i=read(m_fd, msg, m_bufsize); 
      if ( i <= 0 ) 
	{
	  break; 
	}
      else 
	{ 
	  string message = "debug - read "; 
	  message += msg; 
	  m_logger->log(Logger::DEBUG, message); 
	  string read_message( msg ); 
	  string op_str = ""; 
	  string oldname = ""; 
	  string newname = ""; 
	  string::size_type pos; 
	  pos = read_message.find_first_of( " \t" ); 
	  op_str = read_message.substr( 0, pos ); 
	  read_message = read_message.substr( (pos + 1 )); 
	  pos = read_message.find( _changedfiles_seperator );
	  if ( pos == string::npos  )
	    { 
	      newname = "NULL";
	      oldname = read_message;
	    }
	  else
	    {
	      newname = read_message.substr((pos + _changedfiles_seperator.length() )); 
	      oldname = read_message.substr(0, pos);
	    }


	  // deleted by queuehandler
	  // this all needs to move to quehandler & actionqueue
	  FileOperation fo ( op_str, oldname, newname );
	  if(  m_que->add_file( fo ) )
	    {
	      Logger* logger = Logger::get_instance();
	      logger->log( Logger::DEBUG, "BangReader::Adding " + oldname );
	      numread++;
	    }
	}
    }
  return numread; 
}

void BangReader::check_loop() 
{ 
  while ( 1 ) 
    { 
      read_new(); 
      usleep(REST_INTERVAL); 
    }
}

string BangReader::get_error() 
{ 
  return m_error; 
} 

bool BangReader::start()
{ 
  pthread_t *reader_pthreadp; 

  reader_pthreadp = (pthread_t *)malloc( sizeof( pthread_t ) ); 
  pthread_create( reader_pthreadp, NULL, reader_thread_startup, this ); 
  pthread_detach( *reader_pthreadp ); 
  free( reader_pthreadp ); 
  return true; 
}
    
