/*****************************************************************************
     This file is part of Changedfiles system. 

     Copright (C)2001  Michael L. Welles

     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

******************************************************************************/
/*  $Id: file_operation.cpp,v 1.21 2003/12/01 23:51:09 mike Exp $ */
#include "file_operation.h"
#include "bangconfig.h"
#include "bangreader.h"
#include "operations.h"
#include "logger.h" 
#include <string> 
extern "C" { 
#include <stdio.h> 
#include <sys/types.h>
#include <fcntl.h>
#include <error.h> 
#include <errno.h>
#include <unistd.h> 
#include <time.h>
#include <regex.h>
#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h> 
}

using std::string; 


FileOperation::FileOperation( const string op_str, 
			      const string oldfile )
{
  string nullstr = "NULL";
  this->set_members( op_str, oldfile, nullstr );
}

FileOperation::FileOperation( const string op_str, 
		    const string oldfile, 
		    const string newfile )
{ 
  this->set_members( op_str, oldfile, newfile );
}

void FileOperation::set_members(  const string op_str, 
		    const string oldfile, 
		    const string newfile )
{ 
  m_logger = Logger::get_instance();
  time_t now; 
  if ( op_str == "LINK" )
    { 
      set_operation( LINK ); 
    }
  else if ( op_str == "SYMLINK" )
    { 
      set_operation( SYMLINK ); 
    }
  else if ( op_str == "RENAME" )
    { 
      set_operation( RENAME ); 
    }
  else if ( op_str == "OPENW" ) 
    { 
      set_operation( OPENW ); 
    } 
  else if ( op_str == "MKDIR" ) 
    { 
      set_operation( MKDIR ); 
    }
  else if ( op_str == "RMDIR" ) 
    { 
      set_operation( RMDIR ); 
    }
  else if (op_str == "UNLINK" )
    { 
      set_operation( UNLINK ); 
    } 
  else if (op_str == "CREATE" )
    {
      set_operation( CREATE ); 
    } 
  else if (op_str == "CLOSE" ) 
    { 
      set_operation( CLOSE ); 
    }
  else if (op_str == "OPENR" )
    { 
      set_operation( OPENR ); 
    }
  else if ( op_str == "CHMOD" )
    {
      set_operation( CHMOD );
    }
  else if ( op_str == "CHOWN" )
    {
      set_operation( CHOWN );
    }
  else
    {
      set_operation( NOOP );
    }

  // just to clear out the cobwebs...
  m_old_file = m_new_file = m_filename = "";

  string nullstr( "NULL" );
  if( newfile != nullstr )
    {
      set_old_file( oldfile ); 
      set_new_file( newfile ); 
      set_filename( "" );
    }
  else
    {
      set_filename( oldfile ); 
      
      if ( op_str == "OPENW" )
	{ 
	  set_inode(); 
	}
    }

  time( &now ); 
  set_watchtime( now ); 
  m_valid = true; 

}

time_t FileOperation::get_watchtime() 
{ 
  return m_watchtime; 
}

bool FileOperation::set_watchtime( time_t t )
{ 
  m_watchtime=t; 
  return true; 
}

bool FileOperation::set_timeout( int t )
{ 
  m_timeout=t; 
  return true; 
}

bool FileOperation::is_open( ) 
{ 
  DIR* procdir; 
  struct dirent *dir; 
  regex_t preg; 
  string message; 
  if ( regcomp(&preg, "^[0-9]+$", REG_EXTENDED) != 0 )
    { 
      m_error = "error - Error compiling regular expression"; 
      m_logger->log( Logger::ERR, m_error ); 
      return false; 
    }

  procdir = opendir("/proc"); 
  if (procdir  == NULL ) 
    { 
      m_error = "error - Error opening /proc !"; 
      m_logger->log( Logger::ERR, m_error ); 
      return false; 
    }
  while ( (dir = readdir( procdir )) != NULL ) 
    { 

      if ( regexec( &preg, dir->d_name, 0, NULL, 0 ) == 0 )
	{ 
	  message = dir->d_name; 
	  message += " matches"; 
	  // m_logger->log(Logger::DEBUG, message ); 
	  DIR *subdir; 
	  struct dirent *subdirent; 
	  string subdirname; 
	  subdirname = "/proc/"; 
	  subdirname += dir->d_name; 
	  subdirname += "/fd"; 
	  subdir = opendir( subdirname.c_str() ); 
	  if (subdir == NULL ) 
	    { 
	      m_error = "error -  Error opening " + subdirname; 
	      closedir( subdir ); 
	    }
	  else 
	    { 
	      while ( (subdirent=readdir( subdir )) != NULL ) 
		{ 
		  struct stat statbuf; 
		  string filename; 
		  filename = subdirname + "/" + subdirent->d_name; 
		  //m_logger->log( Logger::DEBUG, filename ); 
		  if ( stat( filename.c_str(), &statbuf ) == 0 ) 
		    { 
                        // first compare devices
			if ( statbuf.st_dev == get_dev() )
			{
		           char inode[32];
		           inode[0] = '\0'; 
		           if ( statbuf.st_ino == get_inode() ) 
			   { 
			      regfree(&preg); 
			      closedir( procdir ); 
			      closedir( subdir ); 
			      return true; 
			   }
			}
		    }
		}
	      closedir( subdir ); 
	    }
	}
    }
  regfree(&preg); 
  closedir( procdir ); 
  return false; 
}

bool FileOperation::is_ready()
{ 
  m_logger = Logger::get_instance();
  string message;
  if ( get_operation() != OPENW ) 
    {
      return true;
    }
  else 
    { 
      if ( ! is_open() ) 
	{
	  return true; 
	}
      return false; 
    }
}

bool FileOperation::is_timeout()
{ 
  m_logger = Logger::get_instance();
  string message;
  if ( get_operation() != OPENW ) 
    {
      return true; 
    }
  else 
    { 
      if ( is_open() ) 
	{ 
	  time_t now, then; //, interval; 
	  time(&now); 
	  then=get_watchtime(); 
	  if (now > (then + this->m_timeout)
	      && (this->m_timeout != -1 ))
	    { 
	      // we reset the timer to prepare future timeouts ...
	      set_watchtime(now);
	      m_logger->log( Logger::INFO, "Timeout reached.  File '" + m_filename + "' is still open for write, but attempting action anyway." ); 
	      return true; 
	    }
	}
      return false; 
    }
}
  
ino_t FileOperation::get_inode() 
{ 
  return m_inode; 
}

dev_t FileOperation::get_dev() 
{ 
  //m_dev is set via set_inode()
  return m_dev; 
}

bool FileOperation::set_inode()
{ 
  struct stat statbuf; 
  if (  ( stat( get_filename().c_str(), &statbuf )) == 0 )
    { 
      m_inode = statbuf.st_ino; 
      m_dev   = statbuf.st_dev;
      return true; 
    }
  else 
    { 
      return false; 
    }
}

e_optype FileOperation::get_operation() 
{ 
  return m_operation; 
}

bool FileOperation::set_operation( e_optype operation )
{ 
  m_operation = operation; 
  return true; 
}

string FileOperation::get_old_file()
{ 
  return m_old_file; 
}

bool FileOperation::set_old_file( string filename )
{
  m_old_file = filename; 
  return true; 
}

string FileOperation::get_new_file()
{ 
  return m_new_file; 
}

bool FileOperation::set_new_file( string filename )
{ 
  m_new_file = filename; 
  return true; 
}

string FileOperation::get_filename() 
{ 
  return m_filename; 
}

bool FileOperation::set_filename( string filename )
{ 
  m_filename = filename; 
  return true; 
}

bool FileOperation::is_valid()
{
  string message = "debug - FileOperation::is_valid() debug;  "; 
  if ( m_valid ) 
    { 
      message += "returning true"; 
    }
  else 
    { 
      message += "returning false"; 
    }
  //m_logger->log( Logger::DEBUG, message ); 
  return m_valid; 
}
