#include "action_queue.h"
#include "command_action.h"
//#include "action_exception.h"
#include "ftp_connection_manager.h"
#include "logger.h"
#include <string> 
#include <list> 
#include <map> 
#include <exception>
#include <stdexcept>
#include <pthread.h>
#define  FILE_WHICH_SHOULD_NOT_BE "JUST_CHECKING_THE_CONNECTION"
extern "C"{
#include <errno.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <libfget.h>
}

using std::string; 


FTPConnectionManager * FTPConnectionManager::m_ftp_connection_manager = 0;

namespace
{
  pthread_mutex_t m_ftp_connection_manager_mutex = PTHREAD_MUTEX_INITIALIZER; 
}

FTPConnectionManager::FTPConnectionManager()
{
  m_ftp_connection_manager_instantiated = true; 
  m_cache_expire = -1;
  m_flags = FTP_PASSIVE;
}

FTPConnectionManager* FTPConnectionManager::get_instance()
{
  pthread_mutex_lock( &m_ftp_connection_manager_mutex );
  if ( m_ftp_connection_manager == 0|| m_ftp_connection_manager->is_instantiated() == false )
    {
      m_ftp_connection_manager = new FTPConnectionManager();
    }
  pthread_mutex_unlock( &m_ftp_connection_manager_mutex );
  return m_ftp_connection_manager;
}


FTP * FTPConnectionManager::connect( string hostname, string username, string password )
{
  FTP *ftp;
  int bannersize = 512;
  char banner[bannersize];

  int hostsize = sizeof( hostname.c_str() );
  char host[(hostsize+5)];
  ftp_hookfunc_t hook = NULL;
  string errStr;
  
  strcpy( host, hostname.c_str() );

  bool ret = false;
  int retVal;

  retVal = ftp_connect( &ftp, host, 
			banner, bannersize, this->m_flags, 
			300, this->m_cache, 
			this->m_cache_expire, 
			hook/* sendhook */, hook  /* receivehook */ );

  if( retVal == 0 ) 
    {
      retVal = ftp_login( ftp, (char*)username.c_str(), 
			  (char*)password.c_str() );

      if( retVal == 0 ){ ret = true; }
      else
	{
	  Logger *logger = Logger::get_instance();
	  string message = "FTP login unsucessful ";
	  message += hostname;
	    
	  logger->log( Logger::ERR, message );
	  return NULL;
	}
    }
  else
    {
      Logger *logger = Logger::get_instance();
      string message = "FTP connection to ";
      message += hostname; 
      message += " failed.";
      logger->log( Logger::ERR, message );
      return NULL;
    }
  Logger *logger = Logger::get_instance();
  string message = "Created new FTP connection to ";
  message += hostname;
  logger->log( Logger::INFO, message );
  return ftp;
}

bool FTPConnectionManager::is_instantiated()
{ 
  return m_ftp_connection_manager_instantiated;
}

bool FTPConnectionManager::disconnect( FTP *ftp )
{
  bool ret = false;
  Logger* logger = Logger::get_instance();

  if( ftp == NULL ) {
    return true;
  }
  if( ( ftp_quit( ftp, m_flags ) ) == 0 )
    {
      logger->log( Logger::INFO, "FTPConnectionManager: connection closed" );
      ret = true;
    }else{
      logger->log( Logger::INFO, "FTPConnectionManager: failed to close connection (posibly already closed by remote host)" );
	ret = false; 
    }
  return ret;
}


FTP* FTPConnectionManager::get_connection( string host, string username, string password )
{
  string combo_key = host;
  combo_key += username;
  combo_key += password;

  pthread_mutex_lock( &m_ftp_connection_manager_mutex );

  Logger* logger = Logger::get_instance();
  FTP *connection = m_connection_map[ combo_key ];
  m_connection_map[ combo_key ] = NULL;
  pthread_mutex_unlock( &m_ftp_connection_manager_mutex );

  if ( connection != NULL )
    { 
      //if connection is bad, will close it fast
      if ( check_connection( connection, host ))
	{ 
	  logger->log( Logger::INFO, "Re-using existing ftp connection to " + host );
	  return connection;
	}
    }
  logger->log( Logger::INFO, "Creating new ftp connection to " + host );
  return connect( host, username, password );
}


bool FTPConnectionManager::return_connection( string host, string username, string password, FTP *connection)
{
  string combo_key = host;
  combo_key += username;
  combo_key += password;
  pthread_mutex_lock( &m_ftp_connection_manager_mutex );
  if ( m_connection_map[ combo_key ] == NULL )
    {
      m_connection_map[ combo_key ] = connection;
    }
  else
    {
      ftp_quit( connection, 0 );
    }
  pthread_mutex_unlock( &m_ftp_connection_manager_mutex );
  return true;
}

bool FTPConnectionManager::check_connection( ftp *connection, string hostname )
{
  FTPFILE* ftp_file;
  // Should always fail and set errno....
  int err = ftp_open( &ftp_file, connection, (char *)FILE_WHICH_SHOULD_NOT_BE, O_RDONLY );

  Logger* logger = Logger::get_instance();
  // string message = "ftp check connection returned ";
  // message += strerror(errno);
  //  logger->log( Logger::INFO, message);
  if (err == -1 && 
      errno != ENOENT && 
       errno != EACCES && 
       errno != EPERM)
    {
      string message = "FTP connection to ";
      message += hostname;
      message += " was closed or failed. Reconnecting.";
      logger->log( Logger::INFO, message );
      ftp_quit( connection, FTP_FASTCLOSE );
      return false;
    }
  else
    { 
      return true;
    }
}

  
	  
  

