/*****************************************************************************
     This file is part of the Changedfiles system. 

     Copright (C)2001  Michael L. Welles

     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*******************************************************************************/
/* $Id: logger.cpp,v 1.13 2003/11/13 19:41:16 mike Exp $ */ 
#include <string> 
#include "logger.h"
#include <pthread.h> 
#include <time.h>
#include <stdio.h>
using std::string; 
//using std::cout;

Logger * Logger::m_logger = 0; 

namespace 
{
  pthread_mutex_t m_logger_mutex = PTHREAD_MUTEX_INITIALIZER; 
  pthread_mutex_t m_logger_other_mutex = PTHREAD_MUTEX_INITIALIZER; 
  pthread_mutex_t m_logger_open_mutex = PTHREAD_MUTEX_INITIALIZER; 
} 

Logger::Logger()
{ 
  m_logfile_name = "STDOUT"; 
  m_logfile_file_p = NULL; 
  m_logger_instantiated = true; 
  m_log_level = INFO;
}

Logger::~Logger()
{ 
  close_log(); 
  m_logger_instantiated = false;
  delete m_logger;
}

void Logger::set_log_level( log_type level )
{
  m_log_level = level;
}

bool Logger::open_log()
{ 
  if ( m_logfile_name.length() != 0  
       && m_logfile_name != "STDOUT" )
    { 
      m_logfile_file_p = fopen( m_logfile_name.c_str(), "a+"); 
      if ( m_logfile_file_p == NULL ) 
	{ 
	  return false; 
	} 
    }
      return true;
}

Logger * Logger::get_instance()
{ 
  pthread_mutex_lock( &m_logger_mutex ); 
  if (  m_logger == 0 || m_logger->is_instantiated() == false ) 
    { 
      m_logger = new Logger(); 
    } 
  pthread_mutex_unlock( &m_logger_mutex ); 
  return m_logger; 
}

bool Logger::is_instantiated()
{ 
  return m_logger_instantiated; 
}

void Logger::set_logfile( string log_file ) 
{ 
  m_logfile_name = log_file; 
  
}
void Logger::log( log_type logtype, const char * msg )
{ 
  pthread_mutex_lock( &m_logger_other_mutex ); 
  string message( msg ); 
  log( logtype, message ); 
  pthread_mutex_unlock( &m_logger_other_mutex ); 
}

void Logger::log( log_type logtype, string msg )
{ 
  pthread_mutex_lock( &m_logger_mutex ); 
  time_t epoch_time; 
  struct tm *local_time; 

  char fmt_time[64]; 
  fmt_time[0] = '\0'; 
  if ( logtype >= m_log_level ) 
    { 
      epoch_time = time( NULL ); 
      local_time = localtime( &epoch_time ); 

      strftime( fmt_time, 64, "[%Y/%m/%d %H:%M:%S] ", local_time ); 

      string now_time( fmt_time ); 
      if ( m_logfile_name.length() == 0  || m_logfile_name  == "STDOUT" ||
	   m_logfile_file_p == NULL || m_logfile_name == "" )
	{ 
	  printf( "%s %s. [thread id %u]\n", now_time.c_str(),  msg.c_str(), pthread_self()  );
	}
      else 
	{ 
	  string message = now_time + msg + "\n"; 
	  if (  ! m_logger->open_log() )
	  	{
	  			printf("Problem openeing logfile!, logging to STDOUT\n");
	  			printf( "%s%s\n", now_time.c_str(), msg.c_str()  );
	  	}
	  	else
	  	{			 
			  fprintf( m_logfile_file_p, "%s", message.c_str()); 
		  	  m_logger->close_log(); 
	  	}
	}
    }
  pthread_mutex_unlock( &m_logger_mutex ); 
}

bool Logger::close_log()
{ 
  if ( m_logfile_file_p == NULL ) 
    { 
      return false; 
    }
  if ( ( fclose( m_logfile_file_p ) == 0 ) )
    { 
      return true; 
    }
  else 
    { 
      return false; 
    }
}

    
