/*****************************************************************************
     This file is part of the Changedfiles system

     Copright (C)2001  Michael L. Welles

     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*******************************************************************************/
/*  $Id: main.cpp,v 1.44 2003/11/13 19:41:16 mike Exp $ */
#define _THREAD_SAFE
#include "bangreader.h"
#include "bangconfig.h"
#include "queue_handler.h"
#include "logger.h" 
#include <string> 
#include <iostream>
#include <fstream> 
#include <syslog.h>
#include <signal.h>
#include <stdio.h> 
#include <errno.h> 
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <exception>
#include <stdexcept>

using std::cout; 
using std::cerr; 
using std::endl; 
using std::string; 

void go( string&, bool ); 
void daemonize( bool ); 
void usage(); 

extern "C" { 
  void sig_handler( int, siginfo_t* , void *); 
}

namespace
{ 
  // this ought to be obtained from config.h
  const string DEFAULT_CONF_FILE = "/usr/local/changedfiles/etc/changedfiles.conf"; 
  const int NOFILE = 1024; 
  bool debug = false; 
  bool re_init = false; 
  const int REST_MICROSECONDS = 200; 
}

/* 
 * added this to have one less global variable out there...
 */
void log( Logger::log_type logtype, string msg )
{
  Logger * logger; 
  logger = Logger::get_instance();

  logger->log( logtype, msg );
}

void terminator()
{
  cerr << "Terminal error occurred" << endl;
  exit(1);
}

int main( int argc, char * argv[] ) 
{ 
  int opt; 
  string config_file; 
  string log_file; 

  bool debug=false; 
  
  //void (*old_terminate)()  = set_terminatete(terminator);



  while ( ( opt = getopt( argc, argv, "dc:l:" ) ) != -1 )
    { 
      switch ( opt ) 
	{ 

	case 'd':
	  debug = true; 
	  break;
	 
	case 'c': 
	  config_file = optarg; 
	  break; 

	case 'l': 
	  log_file = optarg; 
	  break; 
	    
	default: 
	  usage();
	  exit(1);
	  break; 
	}
    }

  Logger* logger = Logger::get_instance();

  if( debug )
    {
      log_file = "";
      logger->set_log_level( Logger::DEBUG );
    }
  else
    { 
      
      logger->set_log_level( Logger::INFO );

      if ( log_file.length() == 0 )
	log_file = DEFAULT_LOG_FILE; 

    }

  if ( config_file.length() == 0 )
    { 
      config_file = DEFAULT_CONF_FILE; 
    }

  
  

//   struct sigaction action;
	
//   action.sa_flags = 0;
//   action.sa_sigaction = sig_handler;
//   sigemptyset( &action.sa_mask );
//   if ( sigaction( SIGINT, &action, 0 ) == -1 )
//     {
//       cout << "Can't set SIGINT signal handler" << endl;
//       perror( "sigaction()" );
//       exit( 1 );
//     }
	
//   if ( sigaction( SIGTERM, &action, 0 ) == -1 )
//     {
//       cout << "Can't set SIGTERM signal handler" << endl;
//       perror( "sigaction()" );
//       exit( 1 );
//     }
	
//   if ( sigaction( SIGHUP, &action, 0 ) == -1 )
//     {
//       cout << "Can't set SIGHUP signal handler" << endl;
//       perror( "sigaction()" );
//       exit( 1 );
//     }
	
//   if ( sigaction( SIGPIPE, &action, 0 ) == -1 )
//     {
//       cout << "Can't set SIGPIPE signal handler" << endl;
//       perror( "sigaction()" );
//       return false;
//     }
  Logger* m_logger = Logger::get_instance(); 
  if ( ! debug ) 
    { 
      m_logger->set_logfile( log_file ); 
      if ( !  m_logger->open_log() )
	{ 
	  cerr << "Error! Couldn't open logfile:" << log_file; 
	  exit( 1 ); 
	}
    }
  m_logger->log( Logger::INFO, "############################################################" ); 
  m_logger->log( Logger::INFO, "info  - changedfiles starting" ); 

  while ( 1 ) { 
	go( config_file, debug ); 
  }
}
/** this is where it all happens, for most normal operation */
void go( string &config_file, bool debug )
{ 
  BangConf* config = new BangConf( config_file ); 

  if ( config->is_valid() == false ) 
    { 
      string msg; 
      msg = "error - Error! Config file not valid: " + config->get_error(); 
      log( Logger::ERR, msg); 
      exit(1); 
    }
  if (  debug ) 
    { 
      cout << "Starting changedfiles in debug mode\n";
    }
  else
    {
      daemonize(false); 
    }



  
  QueueHandler handler( config ); 
  BangReader reader( &handler ); 

  delete config;

#ifndef CFDEBUG
  //start reader thread
  reader.start(); 
  //
  // Handler thread spawns child thread for for each action queue
  handler.start(); 
#endif  
  
  while ( 1 ) 
    { 
#ifdef CFDEBUG
      reader.read_new(); 

      usleep( REST_MICROSECONDS ); 

      handler.do_pending(); 
#endif 
      usleep( REST_MICROSECONDS ); 
    }
}

extern "C" {
  void sig_handler( int sig, siginfo_t*, void* )
  {	
    if ( sig == SIGTERM )
      {
	syslog( LOG_ERR, "error - received SIGTERM.  Exiting..." );
	if ( debug )
	  {
	    cout << "Received signal SIGTERM.  Exiting..." << endl;
	  }
	exit( 1 ); 
      }
    else if ( sig == SIGINT )
      {
	syslog( LOG_ERR, "error - received SIGINT.  Exiting..." );
	if ( debug )
	  {
	    cout << "Received signal SIGINT.  Exiting..." << endl;
	  }
	exit( 1 ); 
      }
    else if ( sig == SIGHUP )
      {
	syslog( LOG_WARNING, "warn  - received SIGHUP, Exiting..." );
	if ( debug )
	  {
	    cout << "Received signal SIGHUP.  Reinitializing... " << endl;
	  }
	exit( 1 ); 
      }
    else if ( sig == SIGPIPE )
      {
	syslog( LOG_ERR, "error - received SIGPIPE.  Exiting" );
	if ( debug )
	  {
	    cout << "Received signal SIGPIPE.  Exiting..." << endl;
	  }
	exit( 1 ); 
	
      }
    else
      {
	if ( debug )
	  {
	    cout << "Received unknown signal: " << sig << endl;
	  }
      }	
  }
}

/* 
 * ignsigcld --> if called with nonzero -> handle SIGCLDs 
 * so zombies dont clog. 
 * this code cribbed almost completely from Stevens, UNIX Network
 *  Programming, section 2.6 
 */
void daemonize(bool ignsigcld) {


  register int childpid, fd; 
  FILE *foo; 
  
  if (getppid() == 1) 
    { 
      goto out; 
    }
  
#ifdef SIGTTOU
  signal(SIGTTOU, SIG_IGN); 
#endif 
  
#ifdef SIGTTIN
  signal (SIGTTIN, SIG_IGN); 
#endif
  
#ifdef SIGTSTP
  signal(SIGTSTP, SIG_IGN); 
#endif
  
  if ( (childpid = fork()) < 0 ) 
    { 
      exit(errno);
    } 
  else if (childpid > 0) 
    { 
      // we're off & demonized
      exit(0); 
    }

  /*  
   *   #ifdef SIGTSTP 
   *   if (setpgrp(0, getpid()) == -1)
   *   err_sys("Can't change process group"); 
   *   if ( (fd = open("/dev/tty", O_RDWR)) >=0 ) { 
   *   ioctl(fd, TIOCNOTTY, (char *)NULL); 
   *   close(fd);
   *   }
   *   #else 
   */ 

  if (setpgrp() == -1) 
    { 
      /* err_sys("Can't change process group");  */ 
      exit(errno); 
    }

  signal (SIGHUP, SIG_IGN); 

  if ( (childpid = fork()) < 0 ) 
    /* err_sys("can't fork second child");  */ 
    exit(errno); 
  else if (childpid > 0 ) 
    exit(0); 
  /* #endif */ 

 out: 
  /*
   * redirect stdout, stderr to /dev/null
   * close stdin.
   * close any others. 
   */ 
  
  foo=freopen("/dev/null", "a+", stderr); 
  if ( foo == NULL ) 
    { 
    exit(1);
    }
  foo=freopen("/dev/null", "a+", stdout); 

  if ( foo == NULL ) 
    { 
    exit(1);
    }
  close(0); 
  /*stdin*/ 

  for (fd=3; fd < NOFILE; fd++) 
    close(fd); 
  
  errno = 0; 

  chdir ("/"); 

  umask(0); 
  if (ignsigcld) 
    { 
#ifdef SIGSTP
      int sig_child; 
      signal(SIGCLD, sig_child);
#else
      signal (SIGCLD, SIG_IGN) ;
#endif
    }
}

void usage() 
{ 

  cout << "Usage:" << endl; 
  cout << "changedfiles [-c config-file ] [-d]" << endl; 
  cout << "\toptions:" << endl; 
  cout << "\t\t-c config-file  -- specify alternate config file." << endl; 
  cout << "\t\t-d              -- turn on debugging" << endl; 
  cout << "\t\t-l log-file     -- specify alternate log file." << endl; 

}

