#ifdef _CHANGEDFILES_HAS_MYSQL_
#include "action_queue.h"
#include "command_action.h"
//#include "action_exception.h"
#include "mysql_connection_manager.h"
#include "logger.h"
#include <string> 
#include <list> 
#include <map> 
#include <exception>
#include <stdexcept>
#include <pthread.h>
extern "C"{
#include <errno.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include "/usr/local/mysql/include/mysql/mysql.h"
}

using std::string; 

MySQLConnectionManager * MySQLConnectionManager::m_mysql_connection_manager = 0;

namespace
{
  pthread_mutex_t m_mysql_connection_manager_mutex = PTHREAD_MUTEX_INITIALIZER; 
}

MySQLConnectionManager::MySQLConnectionManager()
{
  m_mysql_connection_manager_instantiated = true; 
}

MySQLConnectionManager* MySQLConnectionManager::get_instance()
{
  pthread_mutex_lock( &m_mysql_connection_manager_mutex );
  if ( m_mysql_connection_manager == 0|| m_mysql_connection_manager->is_instantiated() == false )
    {
      m_mysql_connection_manager = new MySQLConnectionManager();
    }
  pthread_mutex_unlock( &m_mysql_connection_manager_mutex );
  return m_mysql_connection_manager;
}


bool MySQLConnectionManager::connect( const char * hostname, 
				      const char * username, 
				      const char * password, 
				      const char * database,
				      MYSQL *mysql )
{
  Logger *logger = Logger::get_instance();
  mysql_init( mysql );
  mysql_options(mysql, MYSQL_READ_DEFAULT_GROUP, "changedfiles" );
  if ( ! mysql_real_connect( mysql, 
			     hostname, 
			     username, 
			     password, 
			     database, 
			     0, NULL, 0 ))
    {
      string err = "mysql connection manager ould not connect to database: ";
      err += mysql_error( mysql );
      logger->log( Logger::ERR, err );
      return false;
    }
  return true;
}

bool MySQLConnectionManager::is_instantiated()
{ 
  return m_mysql_connection_manager_instantiated;
}

bool MySQLConnectionManager::disconnect( MYSQL *mysql )
{
  if( mysql == NULL ) {
    return true;
  }
  mysql_close( mysql );
  free( mysql );
  return true;
}

MYSQL* MySQLConnectionManager::get_connection( const char *hostname, const char *  username, const char *password, const char *database )
{
  string combo_key = hostname;
  combo_key += username;
  combo_key += password;
  combo_key += database;
  pthread_mutex_lock( &m_mysql_connection_manager_mutex );

  Logger* logger = Logger::get_instance();
  MYSQL *mysql = m_connection_map[ combo_key ];
  m_connection_map[ combo_key ] = NULL;
  pthread_mutex_unlock( &m_mysql_connection_manager_mutex );
  if ( mysql != NULL )
    { 
      if ( mysql_ping( mysql ) == 0 )
	{
	  return mysql;
	}
      else
	{
	  logger->log( Logger::ERR, "MySQL ping failed, attempting to establish new connection.");
	  mysql_close(mysql);
	  free( mysql );
	}
    }
  logger->log( Logger::INFO, "Creating new mysql connection.");
  mysql = (MYSQL *)malloc( sizeof( MYSQL ));
  connect( hostname, username, password, database, mysql );
  return mysql;
}

bool MySQLConnectionManager::return_connection( const char *hostname, 
						const char *username, 
						const char *password, 
						const char *database,
						MYSQL *mysql )
{
  string combo_key = hostname;
  combo_key += username;
  combo_key += password;
  combo_key += database;
  pthread_mutex_lock( &m_mysql_connection_manager_mutex );
  if ( m_connection_map[ combo_key ] == NULL )
    {
      m_connection_map[ combo_key ] = mysql;
    }
  else
    {
      mysql_close( mysql );
      free( mysql );
    }
  pthread_mutex_unlock( &m_mysql_connection_manager_mutex );
  return true;
}

#endif  
	  
  

