/*****************************************************************************
     This file is part of the Changedfiles system 

     Copright (C)2001  Michael L. Welles

     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*******************************************************************************/
/*  $Id: queue_handler.cpp,v 1.35 2003/11/10 21:58:43 mike Exp $ */
#include "queue_handler.h"
#include "rule.h"
#include "action_queue.h"
#include "command_action.h"
#include "action_factory.h"
//#include "action_exception.h"
#include <string> 
using std::string; 

/**
 * this is the only guy that uses BangConf
 */
QueueHandler::QueueHandler(BangConf* config )
{
  RULE_MAP::iterator iter; 
  RULE_MAP rules; 
  rules=config->get_rules(); 
  for ( iter=rules.begin(); iter != rules.end(); ++iter ) 
    { 
      Rule* rule=(*iter).second; 
      ActionQueue* actionqueue = new ActionQueue( rule ) ; 
      m_queue_list.push_back( *actionqueue ); 
    }
}

QueueHandler::~QueueHandler()
{
  QUEUE_LIST::iterator liter; 
  for ( liter=m_queue_list.begin(); liter != m_queue_list.end(); ++liter ) 
    { 
      delete &(*liter);
    }
}

/*
 * launch all the ActionQueue threads
 */
void QueueHandler::start()
{ 
  QUEUE_LIST::iterator liter; 
  for ( liter=m_queue_list.begin(); liter != m_queue_list.end(); ++liter ) 
    { 
      (*liter).start(); 
    }
}

/*
 * Create the Action that will be deleted in ActionQueue::do_next()
 */
bool QueueHandler::add_file(FileOperation bangfile)
{ 
  QUEUE_LIST::iterator iter; 
  ActionFactory* factory = ActionFactory::get_instance();
  bool success = false;

  for (iter=m_queue_list.begin(); iter != m_queue_list.end(); ++iter )
   {
      Rule* rule = (*iter).get_rule();
      if ( rule->operation_matches( &bangfile ) )
	{ 
	  Logger* logger = Logger::get_instance();

	  string message = "QueueHandler: matched ";
	  message += bangfile.get_filename();
	  logger->log(Logger::DEBUG, message ); 
	  bangfile.set_timeout( rule->get_timeout() );
	  try
	    {
	      Action* act = factory->create_action(bangfile, rule);
	      success = (*iter).add_action( act );
	    }
	  catch( ... )
	    {
	      string message = "Action creation failed for " + rule->get_name();
	      logger->log( Logger::ERR, message );
	      success = false;
	    }
	}
    }
  return success; 
}


bool QueueHandler::do_pending()
 {
    QUEUE_LIST::iterator iter; 
    Logger* logger = Logger::get_instance();
    logger->log( Logger::INFO, "QueueHandler::do_pending()" );
    for (iter=m_queue_list.begin(); iter != m_queue_list.end(); ++iter) 
      {
	try
	  {
	    (*iter).do_next(); 
	  }
	catch( ... )
	  {
	    logger->log( Logger::ERR, "QueueHandler caught error" );
	  }
      }
    return true;
 }

