/*****************************************************************************
     This file is part of the Changedfiles system. 

     Copright (C)2001  Michael L. Welles

     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*******************************************************************************/


#ifndef __RULE_H_
#define __RULE_H_
#include <string> 
#include <list> 
#include <regex.h>
#include "operations.h" 
#include "logger.h" 
#include "file_operation.h"


typedef std::list<regex_t> IGNORE_LIST; 

class Rule { 

 public: 
  Rule(); 
  ~Rule(){}; 
  bool set_regexp( regex_t regexp );
  bool set_regexp( string regexp_str ); 
  regex_t get_regexp();
  bool set_retry( int retry );
  int get_retry();
  bool set_timeout( int timeout );
  int get_timeout();

  bool set_command_timeout( int timeout );
  int get_command_timeout(); 

  bool set_timeout_retry( int timeout_retry ){ m_timeout_retry = timeout_retry; return true;}
  int get_timeout_retry(){return m_timeout_retry; };
  bool set_rest_interval( int rest_interval ); 
  int  get_rest_interval(); 
  bool add_ignore( string regexp ); 

  bool set_rule_str( const string command );
  bool set_rule_str( e_optype op, const string command );

  string get_name(){ return m_name; }

  string get_rule_str( e_optype op );

  bool operation_matches ( FileOperation* file );

 protected: 

  bool operation_matches ( e_optype op, const string filename );
  bool file_matches ( const string filename );
  bool is_to_be_ignored( const string filename );
  void set_name( string name ){ m_name = name; };

  string m_openw_rule; 
  string m_openr_rule; 
  string m_mkdir_rule;
  string m_rmdir_rule;
  string m_unlink_rule;
  string m_symlink_rule;
  string m_link_rule;
  string m_close_rule; 
  string m_rename_rule; 
  string m_chmod_rule;
  string m_chown_rule;
  string m_rule_str;

  string m_name;

  regex_t m_regexp; 
  IGNORE_LIST ignore_list; 
  time_t m_timeout; 
  int m_timeout_retry;
  int m_command_timeout;
  Logger *m_logger; 
  int m_retry; 
  
}; 

#endif 

