/*****************************************************************************
     This file is part of the Changedfiles system. 

     Copright (C)2001  Michael L. Welles

     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*******************************************************************************/

/* $Id: changedfiles.c,v 1.42 2003/12/05 17:55:06 mike Exp $ */ 

/* #include <linux/malloc.h> */
#ifdef HAVE_CONFIG_H
#include "../config.h"
#endif

#include <linux/slab.h> 
#include <linux/kernel.h> 
#include <linux/module.h> 
#include <sys/syscall.h> 
#include <linux/fs.h> 
#include <linux/sched.h> 
#include <asm/segment.h> 
#include <asm/uaccess.h> 
#include <linux/string.h> 
#include <linux/types.h> 
#include <linux/smp_lock.h>  
#include <asm/segment.h>
#ifndef LINUX_VERSION_CODE
#  include <linux/version.h> 
#endif 
#include <linux/locks.h>  

#ifndef VERSION_CODE
#  define VERSION_CODE(vers,rel,seq) ( ((vers)<<16) | ((rel)<<8) | (seq) )
#endif

#define CHANGEDFILE_OPENW 0
#define CHANGEDFILE_OPENR 1
#define CHANGEDFILE_RMDIR 2
#define CHANGEDFILE_MKDIR 3
#define CHANGEDFILE_SYMLINK 4
#define CHANGEDFILE_LINK 5
#define CHANGEDFILE_UNLINK 6
#define CHANGEDFILE_CREATE 7
#define CHANGEDFILE_CLOSE 8
#define CHANGEDFILE_RENAME 9 
#define CHANGEDFILE_CHOWN 10
#define CHANGEDFILE_CHMOD 11

#define SEPERATOR "]]]----->>"
/* set the version information, if needed */
#ifdef NEED_VERSION
static char kernel_version[] = UTS_RELEASE;
#endif

#ifndef MIN
#define MIN(a,b)        ((a) < (b) ? (a) : (b))
#endif


/* #define DEBUG 1 */ 
/* ring buffer info */

#define BUF_SIZE        512
#define MAX_PATH        512
char changedfiles_buffer[BUF_SIZE][MAX_PATH];
int cf_buffer_head = 0;
int cf_buffer_tail = 0;



/* device info for the changedfiles device */ 
static int changedfiles_major = 40;


/* address of original syscalls sysc*/

int (*original_open)(const char * filename, int flags, int mode); 
int (*original_rmdir)(const char * pathname); 
int (*original_mkdir)(const char * pathname, int mode); 
int (*original_symlink)(const char * oldname, const char * newname); 
int (*original_link)(const char * oldname, const char * newname);
int (*original_unlink)(const char * pathname); 
int (*original_create)(const char * pathname, int mode);
int (*original_rename)(const char * oldname, const char * newname); 
/* int (*original_close)(unsigned int); */
int (*original_chown)(const char * filename, uid_t user, gid_t group);
int (*original_lchown)(const char * filename, uid_t user, gid_t group);
#ifdef __NR_chown32
int (*original_chown32)(const char * filename, uid_t user, gid_t group);
#endif
#ifdef __NR_lchown32
int (*original_lchown32)(const char * filename, uid_t user, gid_t group);
#endif
int (*original_chmod)(const char* filename, mode_t mode);
/* int (*original_fchown)(unsigned int fd, uid_t user, gid_t group); */
char *read_buf; 

int (*getcwd)(char *buf, unsigned long size);

spinlock_t changedfiles_lock = SPIN_LOCK_UNLOCKED; 
unsigned long lock_flags;

int my_strlen ( const char * mstr) 
{ 
  char * c; 
  int i=0; 
  c = (char *)mstr; 
  while ( *c != '\0' ) 
	{ 
	  c++; 
	  i++; 
	}
  return i; 
}

#if LINUX_VERSION_CODE < VERSION_CODE(2,4,0)

int getcwd(char *buf, unsigned long size)
{
	int error;
	struct dentry *pwd = current->fs->pwd;
	*buf='\0'; 
	error = -ENOENT;
	/* Has the current directory has been unlinked? */
	if (pwd->d_parent == pwd || !list_empty(&pwd->d_hash)) {
		char *page = (char *) __get_free_page(GFP_USER);
		error = -ENOMEM;
		if (page) {
			unsigned long len;

			char * cwd = d_path(pwd, page, PAGE_SIZE);
			error = -ERANGE;
			len = PAGE_SIZE + page - cwd;
			if (len <= size) {
				error = len;
				strncat(buf, cwd, size); 
			}
			free_page((unsigned long) page);
		}
	}
	return error;
}
#endif

	

int changedfiles_get_from_buffer(char * c, int size) 
{
  spin_lock_irqsave(&changedfiles_lock, lock_flags); 
  *c='\0'; 

    if (cf_buffer_head == cf_buffer_tail) 
      { 
	 spin_unlock_irqrestore( &changedfiles_lock, lock_flags ) ; 
       return 0;
     }
   strncpy(c,(char *)changedfiles_buffer[cf_buffer_head],size);
   /* kfree(changedfiles_buffer[cf_buffer_head]) */; 
   memset( changedfiles_buffer[cf_buffer_head], 0, MAX_PATH );
   cf_buffer_head++;
   if(cf_buffer_head == BUF_SIZE) cf_buffer_head=0;

   spin_unlock_irqrestore( &changedfiles_lock, lock_flags ); 
   return 0;
}
void changedfiles_malloc_and_zero_buffer () 
{ 
  int i; 
  
  for (i=0; i < BUF_SIZE; i++)
    { 
  /* Pulling this out temporarily, 
   *   Might as well statically allocate.
   *   changedfiles_buffer[i]= kmalloc( MAX_PATH, GFP_KERNEL); */
      memset(changedfiles_buffer[i], 0, MAX_PATH); 
    }
  return; 
}

void changedfiles_free_buffer()
{
  ;
  /* Pulling this out temporarily, 
   *   Might as well statically allocate.
   *int i;
   * for (i=0; i < BUF_SIZE; i++ )
  *   {
  *   kfree(changedfiles_buffer[i]);
  *   }
  */
}
	
int changedfiles_put_to_buffer(char * c)
{
  /*  int len;  */
#ifdef DEBUG
  printk("changedfiles: put_to_buffer: %s\n", c); 
#endif
  spin_lock_irqsave(&changedfiles_lock, lock_flags); 
  if( (cf_buffer_tail + 1) == cf_buffer_head
	 || (
		 ((cf_buffer_tail + 1) == BUF_SIZE)
		 && ( cf_buffer_head == 0 ) ))
    { 
      cf_buffer_head++;
      if ( cf_buffer_head == BUF_SIZE ) 
		{ 
		  cf_buffer_head = 0; 
		}
      /*signal changedfiles_buffer overrun*/
      strncpy( (char *)changedfiles_buffer[cf_buffer_head], "!", 2);
    }
  /* len = my_strlen(c); 
     len++;  */
  if ( ! changedfiles_buffer[cf_buffer_tail] ) 
    { 
      spin_unlock_irqrestore( &changedfiles_lock, lock_flags ); 
      return -ENOMEM; 
    }
  strncpy( (char *)changedfiles_buffer[cf_buffer_tail], c, MAX_PATH); 
  cf_buffer_tail++;

  if ( cf_buffer_tail == BUF_SIZE ) 
    cf_buffer_tail=0;
  spin_unlock_irqrestore( &changedfiles_lock, lock_flags );
  return 1;
}

void changedfiles_log_filename( const char * oldname, const char * newname, const int operation) 
{ 
  mm_segment_t oldfs;
  char msg[MAX_PATH * 2 + 20]; 
  char fulloldname[MAX_PATH]; 
  char fullnewname[MAX_PATH]; 
  long len=0; 

  fulloldname[0]='\0'; 
  fullnewname[0]='\0'; 
  msg[0]='\0';
  
  /* basename file names, fill path if neccessary */ 
  if ( *oldname == '/' ) 
    { 
      strncpy(fulloldname, oldname, MAX_PATH); 
    }
  else 
    { 
      
      /* printk( "pregetcwd: %s\n", fulloldname); */
      oldfs = get_fs(); set_fs( KERNEL_DS );
      len=getcwd(fulloldname, MAX_PATH);
      set_fs( oldfs );
      /* printk( "postgetcwd: %s\n", fulloldname); */
      if ( len <  0 ) 
	{ 
	  printk("changedfiles: getcwd returned an error (%ld)\n", len); 
	  return; 
	}

      if ( ( my_strlen( fulloldname ) + my_strlen( oldname )) >= MAX_PATH )
	{ 
	  len = ( my_strlen( fulloldname ) + my_strlen( oldname )); 
	  printk("changedfiles: Full filename too long! %d bytes.  Max: %d", (int)len, MAX_PATH); 
	  return ; 
	}

      strcat(fulloldname, "/"); 
      strcat(fulloldname, oldname); 
    }

#ifdef DEBUG
  if ( len > 0 ) 
    printk("changedfiles: full oldpath is '%s'\n", fulloldname); 
#endif

  if ( newname != NULL ) 
    { 
      if ( *newname == '/' )
	{ 
	  strncpy( fullnewname, newname, MAX_PATH ); 
	} 
      else 
	{ 
	  oldfs = get_fs(); set_fs( KERNEL_DS );
	  len = getcwd(fullnewname, MAX_PATH); 
	  set_fs( oldfs );
	  if ( len < 0 )
	    { 
	      printk("changedfiles: getcwd return an error (%ld)", len); 
	      return; 
	    }
	  
	  if ( ( my_strlen( fullnewname ) + my_strlen( newname )) >= MAX_PATH )
	    { 
	      len = ( my_strlen( fulloldname ) + my_strlen( oldname )); 
	      printk("changedfiles: Full filename too long! %d bytes.  Max: %d", (int)len, MAX_PATH); 
	      return ; 
	    }


	  strcat(fullnewname, "/"); 
	  strcat(fullnewname, newname); 
	}
#ifdef DEBUG
      if ( len > 0 ) 
	printk("changedfiles: full newpath is '%s'\n", fullnewname); 
#endif
      
    }
  switch ( operation ) 
	{
	case CHANGEDFILE_OPENW:
	  strcpy(msg, "OPENW ");
	  strcat( msg, fulloldname ); 
	  break; 
	  
	case CHANGEDFILE_OPENR: 
	  strcpy(msg, "OPENR ");
	  strcat(msg, fulloldname); 
	  break; 

	case CHANGEDFILE_RMDIR: 
	  strcpy(msg, "RMDIR "); 
	  strcat(msg, fulloldname); 
	  break; 
	  
	case CHANGEDFILE_MKDIR: 
	  strcpy(msg, "MKDIR "); 
	  strcat(msg, fulloldname); 
	  break; 
	  
	case CHANGEDFILE_SYMLINK: 
	  strcpy(msg, "SYMLINK "); 
	  strcat(msg, oldname); 
	  strcat(msg, SEPERATOR); 
	  strcat(msg, fullnewname); 
	  break; 

	case CHANGEDFILE_LINK: 
	  strcpy(msg, "LINK "); 
	  strcat(msg, oldname); 
	  strcat(msg, SEPERATOR); 
	  strcat(msg, fullnewname); 
	  break; 

	case CHANGEDFILE_UNLINK: 
	  strcpy(msg, "UNLINK "); 
	  strcat(msg, fulloldname); 
	  break; 
	  
	case CHANGEDFILE_CREATE: 
	  strcpy(msg, "CREATE "); 
	  strcat(msg, fulloldname); 

	case CHANGEDFILE_CLOSE: 
	  strcpy(msg, "CLOSE "); 
	  strcat(msg, fulloldname); 
	  break; 
	  
	case CHANGEDFILE_RENAME: 
	  strcpy(msg, "RENAME "); 
	  strcat(msg, fulloldname); 
	  strcat(msg, SEPERATOR); 
	  strcat(msg, fullnewname); 
	  break; 
	  
	case CHANGEDFILE_CHOWN:
	  strcpy(msg, "CHOWN ");
	  strcat(msg, fulloldname);
	  break;

	case CHANGEDFILE_CHMOD:
	  strcpy(msg, "CHMOD ");
	  strcat(msg, fulloldname);;
	  break;


	default: 
	  msg[0]='\0'; 
	}

  if ( msg[0] != '\0' ) 
    { 
      changedfiles_put_to_buffer(msg); 
    }


}

int changedfiles_rename( const char * oldname, const char * newname ) 
{ 
  int ret; 
#if DEBUG > 1
  printk("changedfiles: changedfile_rename( %s, %s )\n", oldname, newname); 
#endif
  ret = original_rename( oldname, newname ); 
  if ( ret >= 0 ) 
    { 
#if DEBUG
      printk("changedfiles: changedfile_rename  %s -> %s\n", oldname, newname); 
#endif       
      changedfiles_log_filename( oldname, newname, CHANGEDFILE_RENAME ); 
    }
  return ret; 
}
    
int changedfiles_open( const char * filename, int flags, int mode)
{
  int ret; 
#if DEBUG > 1
  printk("changedfiles: changedfiles_open(%s, %d, %d)\n", 
	 filename, flags, mode); 
#endif 
  ret=original_open( filename, flags, mode); 
  if ( ret >= 0 
       && ( flags & O_WRONLY ||
	    flags & O_RDWR ))
	{ 
#ifdef DEBUG
	  printk("changedfiles: %s opened for write\n", filename); 
#endif
	  changedfiles_log_filename(filename, NULL, CHANGEDFILE_OPENW); 

	}

#ifdef SHOW_READS
  else if ( ret >= 0 )
    {
      /* we'd like to not have a loop where a reader program 
	 gets reports about it's reads */ 
      if ( (strcmp(filename, "/proc/changedfiles" )) != 0  )
      {
#ifdef DEBUG
	printk("changedfiles: %s opened for read\n", filename); 
#endif
	changedfiles_log_filename(filename,  NULL, CHANGEDFILE_OPENR); 
      }
    }
#endif 

#if DEBUG > 1
  printk("changedfiles: changedfiles_open():  returning %d\n", ret); 
#endif

  return ret; 

}


/* static int changedfiles_close( unsigned int fd )  */
/* { */
/*   struct file *f=NULL;  */
/*   int ret;  */

/* #ifdef DEBUG */
/*   printk("changedfiles: changedfiles_close(%d)\n", fd);  */
/* #endif  */

/*   f=current->files->fd[fd];  */

/*   if ( f != NULL &&f->f_mode & FMODE_WRITE )  */
/* 	{  */
/* #ifdef DEBUG */
/* 	  printk("changedfiles: ");  */
/* 	  print_to_root(f->f_dentry);  */
/* 	  printk(" closed\n");  */
/* #endif */
/* 	  changedfiles_log_filename(f->f_dentry->d_name.name,  */
/* 				    CHANGEDFILE_CLOSE);  */

/* 	} */

/*   ret=original_close(fd);  */

/* #ifdef DEBUG */
/*   printk("changedfiles: original_close(%d) returned %d\n", fd, ret);  */
/* #endif */

/*   return ret;  */
/* } */
  



extern int changedfiles_rmdir( const char * pathname )
{ 
  int ret; 
#if DEBUG > 1
  printk("changedfiles: changedfiles_rmdir(%s)\n", pathname); 
#endif 
  ret = original_rmdir( pathname ); 
  if ( ret >= 0 ) 
    { 
#ifdef DEBUG
      printk("changedfiles: %s rmdir\n", pathname); 
#endif
      changedfiles_log_filename(pathname, NULL, CHANGEDFILE_RMDIR); 

    }
  return ret; 
}

extern int changedfiles_mkdir( const char * pathname, int mode)
{
  int ret; 
#if DEBUG > 1
  printk("changedfiles: changedfiles_mkdir(%s, %d)\n", 
	 pathname, mode); 
#endif 
  ret = original_mkdir( pathname, mode); 
  if ( ret >= 0 ) 
	{ 
#ifdef DEBUG
	  printk("changedfiles: %s mkdir\n", pathname); 
#endif
	  changedfiles_log_filename(pathname, NULL, CHANGEDFILE_MKDIR);
	}
  return ret; 
}

extern int changedfiles_symlink(const char * oldname, 
				const char * newname)
{ 
  int ret; 
#if DEBUG > 1
  printk("changedfiles: changedfiles_symlink(%s,%s)\n", 
	 oldname, newname); 
#endif 
  ret = original_symlink( oldname, newname ); 
  if ( ret >= 0 ) 
	{ 
#ifdef DEBUG
	  printk("changedfiles: %s -> %s  symlink\n", oldname, newname);
#endif
	  changedfiles_log_filename(oldname, newname, 
				    CHANGEDFILE_SYMLINK); 
	}
  return ret; 
}


extern int changedfiles_link( const char * oldname, 
			      const char * newname )
{
  int ret; 
#if DEBUG > 1
  printk("changedfiles: changedfiles_link(%s,%s)\n", 
	 oldname, newname); 
#endif 
  ret = original_link(oldname, newname); 
  if ( ret >= 0 ) 
	{ 
#ifdef DEBUG
	  printk("changedfiles: %s -> %s link\n", oldname, newname); 
#endif
	  changedfiles_log_filename(newname, oldname, 
				    CHANGEDFILE_LINK); 
	}
  return ret; 
}

extern int changedfiles_unlink( const char * pathname )
{ 
  int ret;
#if DEBUG > 1
  printk("changedfiles: changedfiles_unlink(%s)\n", 
	 pathname); 
#endif 
  ret = original_unlink( pathname ); 
  if ( ret >= 0 ) 
	{ 
#ifdef DEBUG
	  printk("changedfiles: %s unlink\n", pathname); 
#endif
	  changedfiles_log_filename(pathname, NULL, 
				    CHANGEDFILE_UNLINK); 
	}
  return ret; 
}


extern int changedfiles_create ( const char *pathname, int mode)
{ 
  int ret; 
#if DEBUG > 1
  printk("changedfiles: changedfiles_create(%s,%d)\n", 
  pathname, mode); 
#endif 
  ret = original_create( pathname, mode ); 
  if ( ret >= 0 ) 
	{
#ifdef DEBUG
	  printk("changedfiles: %s create\n", pathname); 
#endif
	  changedfiles_log_filename(pathname, NULL, 
				    CHANGEDFILE_CREATE); 
	}
  return ret; 
}

extern int changedfiles_chown( const char * filename, uid_t user, gid_t group )
{
  int ret;
  ret = original_chown( filename, user, group );
  if ( ret >= 0 )
    {
#ifdef DEBUG
	  printk("changedfiles: %s chown\n", filename); 
#endif
	  changedfiles_log_filename(filename, NULL, 
				    CHANGEDFILE_CHOWN); 
	}
  return ret;  
}

#ifdef __NR_chown32
extern int changedfiles_chown32( const char * filename, uid_t user, gid_t group )
{
  long ret;
  ret = original_chown32( filename, user, group );
  if ( ret >= 0 )
    {
#ifdef DEBUG
	  printk("changedfiles: %s chown\n", filename); 
#endif
	  changedfiles_log_filename(filename, NULL, 
				    CHANGEDFILE_CHOWN); 
	}
  return ret;  
}
#endif

extern int changedfiles_lchown( const char * filename, uid_t user, gid_t group )
{
  int ret;
  ret = original_lchown( filename, user, group );
  if ( ret >= 0 )
    {
#ifdef DEBUG
	  printk("changedfiles: %s chown\n", filename); 
#endif
	  changedfiles_log_filename(filename, NULL, 
				    CHANGEDFILE_CHOWN); 
	}
  return ret;  
}


#ifdef __NR_lchown32
extern int changedfiles_lchown32( const char * filename, uid_t user, gid_t group )
{
  int ret;
  ret = original_lchown32( filename, user, group );
  if ( ret >= 0 )
    {
#ifdef DEBUG
	  printk("changedfiles: %s chown\n", filename); 
#endif
	  changedfiles_log_filename(filename, NULL, 
				    CHANGEDFILE_CHOWN); 
	}
  return ret;  
}
#endif

extern int changedfiles_chmod( const char * filename, mode_t mode )
{
  int ret;
  ret = original_chmod( filename, mode );
  if ( ret >= 0 )
    {
#ifdef DEBUG
	  printk("changedfiles: %s chmod\n", filename); 
#endif
	  changedfiles_log_filename(filename, NULL, 
				    CHANGEDFILE_CHMOD); 
    }
  return ret;
}

int changedfiles_open_dev( struct inode *in, 
				  struct file * fi )
{ 
  MOD_INC_USE_COUNT;
#if DEBUG > 1
  printk("changedfiles: changedfiles_open_dev()\n"); 
#endif 
  return 0; 
}



int changedfiles_close_dev( struct inode *in, 
				   struct file * fi )
{

#if DEBUG > 1
  printk("changedfiles: changedfiles_close_dev()\n"); 
#endif
  MOD_DEC_USE_COUNT; 
  return 0; 
}



/*ssize_t changedfiles_read_dev( struct inode *in, 
				  struct file *fi,
				  char * buf, 
				  unsigned long count ) */
ssize_t changedfiles_read_dev( struct file *filep, char *buf, size_t count, 
			       loff_t *f_pos )
{ 

   int len; 
   int mycount; 



   if ( count > MAX_PATH ) 
     { 
       mycount = MAX_PATH; 
     }
   else 
     { 
       mycount = count; 
     }

   read_buf = kmalloc( mycount, GFP_KERNEL); 

   if ( !read_buf ) { 
     return -ENOMEM; 
   }
   memset(read_buf, 0, mycount); 
   changedfiles_get_from_buffer(read_buf, mycount);
   len = my_strlen(read_buf); 
   if ( len == 0 ) 
     { 
       kfree(read_buf); 
       return 0; 
     }

#ifdef DEBUG
   printk("changedfiles: got '%s' from buffer\n", read_buf); 
#endif
   /* include the \0 */ 
   len++; 
   if (copy_to_user(buf, read_buf, mycount) )
     { 
       printk("changedfiles:  copy_to_user failed\n"); 
       kfree(read_buf);
       return -EFAULT; 
     }
#ifdef DEBUG
   printk("changedfiles: changedfiles_read_dev() returning '%s'\n", read_buf); 
#endif
   kfree(read_buf);
   return len; 
}


static struct file_operations changedfiles_fop = { 
  read: changedfiles_read_dev, 
  open: changedfiles_open_dev, 
  release: changedfiles_close_dev, 
}; 


  
int init_module()
{
  extern long sys_call_table[]; 
  printk("changedfiles: init\n"); 
  original_open = (int (*)(const char *, int, int))(sys_call_table[__NR_open]); 
  original_rmdir = (int (*)(const char *))(sys_call_table[__NR_rmdir]); 
  original_mkdir = (int (*)(const char *, int))(sys_call_table[__NR_mkdir]); 
  original_symlink = (int (*)(const char *, const char *))(sys_call_table[__NR_symlink]); 
  original_link = (int (*)(const char *, const char *))(sys_call_table[__NR_link]);
  original_unlink = (int (*)(const char *))(sys_call_table[__NR_unlink]); 
#ifdef __NR_creat
  original_create = (int (*)(const char *, int))(sys_call_table[__NR_creat]);  
#endif
  original_rename = (int (*)(const char *, const char *))(sys_call_table[__NR_rename]);
  original_chown = (int (*)(const char *, uid_t, gid_t))(sys_call_table[__NR_chown]);
#ifdef __NR_chown32
  original_chown32 = (int (*)(const char *, uid_t, gid_t))(sys_call_table[__NR_chown32]);
#endif
  original_lchown = (int (*)(const char *, uid_t, gid_t))(sys_call_table[__NR_lchown]);
#ifdef __NR_lchown32
  original_lchown32 = (int (*)(const char *, uid_t, gid_t))(sys_call_table[__NR_lchown32]);
#endif
  original_chmod = (int (*)(const char *, mode_t))(sys_call_table[__NR_chmod]);
  /* original_fchown = (int (*)(const char *, uid_t, gid_t))(sys_call_table[__NR_fchown]);*/
  /* original_close = (int (*)(unsigned int))(sys_call_table[__NR_close]); */
  sys_call_table[__NR_open] = (unsigned long)changedfiles_open; 
  sys_call_table[__NR_rmdir] = (unsigned long)changedfiles_rmdir;
  sys_call_table[__NR_mkdir] = (unsigned long)changedfiles_mkdir; 
  sys_call_table[__NR_symlink] = (unsigned long)changedfiles_symlink; 
  sys_call_table[__NR_link] = (unsigned long)changedfiles_link; 
  sys_call_table[__NR_unlink] = (unsigned long)changedfiles_unlink; 
#ifdef __NR_creat
  sys_call_table[__NR_creat] = (unsigned long)changedfiles_create; 
#endif
  sys_call_table[__NR_rename] = (unsigned long)changedfiles_rename; 
  sys_call_table[__NR_chown] = (unsigned long)changedfiles_chown;
#ifdef __NR_chown32
  sys_call_table[__NR_chown32] = (unsigned long)changedfiles_chown32;
#endif
  sys_call_table[__NR_lchown] = (unsigned long)changedfiles_lchown;
#ifdef __NR_lchown32
  sys_call_table[__NR_lchown32] = (unsigned long)changedfiles_lchown32;
#endif
  sys_call_table[__NR_fchown] = (unsigned long)changedfiles_lchown;
  sys_call_table[__NR_chmod] = (unsigned long)changedfiles_chmod;
  /*  sys_call_table[__NR_close] = (unsigned long)changedfiles_close;  */

#if LINUX_VERSION_CODE >= VERSION_CODE(2,4,0)
  getcwd = (int (*)(char * , unsigned long ))(sys_call_table[__NR_getcwd]);
#endif
  changedfiles_malloc_and_zero_buffer(); 

  if ( register_chrdev(changedfiles_major, "changedfiles", &changedfiles_fop))
	{ 
	  return -EIO; 
	}
  return 0; 
}

void cleanup_module()
{ 
  extern long sys_call_table[]; 
  spin_lock_irqsave(&changedfiles_lock, lock_flags); 
  printk("changedfiles: unloading -- resetting symbol table\n"); 
  sys_call_table[__NR_open] = (unsigned long)original_open; 
  sys_call_table[__NR_rmdir] = (unsigned long)original_rmdir; 
  sys_call_table[__NR_mkdir] = (unsigned long)original_mkdir; 
  sys_call_table[__NR_symlink] = (unsigned long)original_symlink; 
  sys_call_table[__NR_link] = (unsigned long)original_link; 
  sys_call_table[__NR_unlink] = (unsigned long)original_unlink; 
#ifdef __NR_creat
  sys_call_table[__NR_creat] = (unsigned long)original_create; 
#endif
  sys_call_table[__NR_rename] = (unsigned long)original_rename;
  sys_call_table[__NR_chown] = (unsigned long)original_chown;
#ifdef __NR_chown32
  sys_call_table[__NR_chown32] = (unsigned long)original_chown32;
#endif
  sys_call_table[__NR_lchown] = (unsigned long)original_lchown;
#ifdef __NR_lchown32
  sys_call_table[__NR_lchown32] = (unsigned long)original_lchown32;
#endif
  /*  sys_call_table[__NR_close] = (unsigned long)original_close; */
  sys_call_table[__NR_chmod] = (unsigned long)original_chmod;
  changedfiles_free_buffer();
  spin_unlock_irqrestore( &changedfiles_lock, lock_flags ) ; 
  unregister_chrdev(changedfiles_major, "changedfiles"); 
}

	
MODULE_AUTHOR("Michael L. Welles <mike@bangstate.com>");
MODULE_DESCRIPTION("monitors and reports file actions to /dev/changedfiles");
MODULE_LICENSE("GPL");
